"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNodeResolveMiddleware = createNodeResolveMiddleware;

var _utils = require("../utils/utils.js");

var _messageChannel = require("../utils/message-channel.js");

var _constants = require("../constants.js");

var _resolveModuleImports = require("../utils/resolve-module-imports.js");

/**
 * @typedef {object} NodeResolveMiddlewareConfig
 * @property {string} rootDir
 * @property {string[]} extraFileExtensions
 * @property {string[]} moduleDirectories
 * @property {boolean} preserveSymlinks
 * @property {boolean} cacheCompilation
 * @property {boolean} logCompileErrors
 */

/**
 * @param {NodeResolveMiddlewareConfig} cfg
 */
function createNodeResolveMiddleware(cfg) {
  const fileExtensions = [..._constants.baseFileExtensions, ...cfg.extraFileExtensions];
  /** @type {import('koa').Middleware} */

  async function nodeResolveMiddleware(ctx, next) {
    const baseURL = ctx.url.split('?')[0].split('#')[0];

    if (baseURL.startsWith('/polyfills') || !fileExtensions.some(ext => baseURL.endsWith(ext))) {
      return next();
    }

    await next(); // should be a 2xx response

    if (ctx.status < 200 || ctx.status >= 300) {
      return undefined;
    }

    const filePath = (0, _utils.getRequestFilePath)(ctx, cfg.rootDir);
    const lastModified = ctx.response.headers['last-modified'];

    if (!filePath || !lastModified) {
      return undefined;
    } // Ensure we respond with js content type


    ctx.response.set('content-type', 'text/javascript');

    try {
      const source = await (0, _utils.getBodyAsString)(ctx);
      const compiled = await (0, _resolveModuleImports.resolveModuleImports)(cfg.rootDir, filePath, source, {
        fileExtensions,
        moduleDirectories: cfg.moduleDirectories,
        preserveSymlinks: cfg.preserveSymlinks
      });
      ctx.body = compiled;
      return undefined;
    } catch (error) {
      const errorMessage = `Error resolving imports in "${ctx.url}": ${error.message}`; // for regular syntax errors we don't send a 500 so that we don't overwrite the
      // browser's syntax error logging

      if (!(error instanceof _resolveModuleImports.ResolveSyntaxError)) {
        ctx.body = errorMessage;
        ctx.status = 500;
      }

      (0, _messageChannel.sendMessageToActiveBrowsers)('error-message', JSON.stringify(errorMessage));

      if (cfg.logCompileErrors) {
        /* eslint-disable-next-line no-console */
        console.error(errorMessage);
      }
    }

    return undefined;
  }

  return nodeResolveMiddleware;
}